#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2010  Edmundo Álvarez Jiménez
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Edmundo Álvarez Jiménez <e.alvarezj@gmail.com>

import pygtk
pygtk.require('2.0')
import gtk
gtk.gdk.threads_init()

import gnomeapplet
import os
import gobject

import threading

import config
from about import AboutDialog
from prefs import PrefsDialog
from overview import OverviewDialog
from launchers import SquealerLauncher
from tempusfugit_mgr import TempusFugitMgr
from Squealer.settings import SettingsReader

class DetrasApplet (object):
    TIMEOUT_INTERVAL = 1 * 1000
    
    def __init__ (self, applet):
        self.applet = applet
        
        self.applet.about = None
        self.applet.preferences = None
        
        self.settings = SettingsReader ()
        
        self.about = None
        self.preferences = None
        self.overview = None
        
        self.squealer = None
        self.tf = TempusFugitMgr (self)
        
        if self.settings.get_start_init ():
            self.tf.launch ()
        
        self.icon = gtk.Image ()
        self.update_icon ()
        self.icon.show ()
        
        self.applet.add (self.icon)
        
        self.squealer_lock = threading.Lock ()
        
        self.applet.connect ("button-press-event", self.on_button_press_cb)
        self.applet.connect ("destroy", self.destroy)
        
        gobject.timeout_add (DetrasApplet.TIMEOUT_INTERVAL, self.timeout_cb)
        
        self.applet.setup_menu_from_file (os.path.join (config.DATA_DIR, \
                "detras-applet"), \
                "DetrasApplet.xml", \
                None, \
                [("about", self.about_cb), \
                ("preferences", self.prefs_cb), \
                ("overview", self.overview_cb), \
                ("upload_data", self.upload_data_cb), \
                ("toggle_tf", self.toggle_tf_cb)])
        
        #self.update_menu ("start_tf", 0)
    
    def destroy (self, event):
        if self.preferences and self.preferences.is_opened ():
            if self.settings.get_stop_exit ():
                self.tf.stop ()
        else:
            settings = SettingsReader ()
            if settings.get_stop_exit ():
                self.tf.stop ()
        
        del (self.applet)
    
    def timeout_cb (self):
        self.update_icon ()
        return 1
    
    def update_menu (self, menu_verb, value):
        popup = self.applet.get_popup_component()
        val = {False: "0", True: "1"}[value]
        
        popup.set_prop("/commands/"+menu_verb, "hidden", val)
        #print popup.get_prop("/commands/"+menu_verb, "name")
    
    def update_icon (self):
        if self.tf.get_status () is TempusFugitMgr.RUNNING:
            icon_path = os.path.join (config.ICONS_DIR, "detras-tracking.png")
            self.applet.set_tooltip_text \
                    ("TempusFugit is tracking your activity")
        else:
            icon_path = os.path.join (config.ICONS_DIR, \
                    "detras-no-tracking.png")
            self.applet.set_tooltip_text ("TempusFugit is stopped")
        
        self.icon.set_from_file (icon_path)
    
    def on_button_press_cb (self, widget, event):
        if event.button == 1:
            self.toggle_tf_cb (None, None)
    
    def upload_finished_cb (self):
        gobject.idle_add (self.__show_confirm, "Upload successful",
                "Your data have been uploaded successfully.")
        
        self.squealer_lock.acquire ()
        self.squealer = None
        self.squealer_lock.release ()
    
    def about_cb (self, component, verb):
        if self.about and self.about.is_opened ():
            self.applet.about.present ()
        else:
            self.about = AboutDialog ()
            self.about.show ()
            
            self.applet.about = self.about.get_about ()
    
    def prefs_cb (self, component, verb):
        if self.preferences and self.preferences.is_opened ():
            self.applet.preferences.present ()
        else:
            self.preferences = PrefsDialog ()
            self.preferences.show ()
            
            self.applet.preferences = self.preferences.get_window ()
    
    def overview_cb (self, component, verb):
        if self.overview and self.overview.is_opened ():
            self.overview.get_window ().present ()
        else:
            self.overview = OverviewDialog ()
            self.overview.show ()
    
    def upload_data_cb (self, component, verb):
        self.squealer_lock.acquire ()
        
        if self.squealer is None:
            self.squealer = SquealerLauncher (self)
            self.squealer.start ()
        
        self.squealer_lock.release ()
    
    def toggle_tf_cb (self, component, verb):
        if self.tf.get_status () is TempusFugitMgr.RUNNING:
            self.tf.stop ()
        else:
            self.tf.launch ()
   
    def error_cb (self, title, message):
        gobject.idle_add (self.__show_error, title, message)
    
    def __show_confirm (self, title, message):
        gtk.gdk.threads_enter ()
        
        msg = gtk.MessageDialog (None,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_INFO, gtk.BUTTONS_OK, message)
        
        msg.set_title (title)
        
        msg.run ()
        msg.destroy ()
        
        gtk.gdk.threads_leave ()
    
    def __show_error (self, title, message):
        gtk.gdk.threads_enter ()
        
        msg = gtk.MessageDialog (None,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, message)
        
        msg.set_title (title)
        
        msg.run ()
        msg.destroy ()
        
        gtk.gdk.threads_leave ()

