#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2010  Edmundo Álvarez Jiménez
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Edmundo Álvarez Jiménez <e.alvarezj@gmail.com>

import subprocess
import threading
import os
import config

class Launcher (threading.Thread):
    """Generic class to control commands execution"""
    def __init__ (self, parent):
        threading.Thread.__init__(self)
        self.parent = parent
    
    def launch (self):
        pass
    
    def run (self):
        self.launch ()


class SquealerLauncher (Launcher):
    """Handle Squealer execution"""
    
    def __init__ (self, parent):
        Launcher.__init__ (self, parent)
        self.popen = None
    
    def launch (self):
        try:
            path = os.path.join (config.BIN_DIR, 'squealer')
            self.popen = subprocess.Popen ([path], stderr=subprocess.PIPE)
            
            stderr = self.popen.communicate (None) [1]
            
            if (stderr is not None) and (stderr != ""):
                self.parent.error_cb ("Squealer error", stderr)
        except (OSError, ValueError), e:
            self.parent.error_cb ("Error", \
                    "Cannot execute Squealer: \n %s" % str(e))
        except:
            self.parent.error_cb ("Error", \
                    "Cannot execute Squealer: \n Unexpected error")
        finally:
            self.parent.upload_finished_cb ()
    

class TempusFugitLauncher (Launcher):
    """Handle TempusFugit execution"""
    
    def __init__ (self, parent):
        Launcher.__init__ (self, parent)
        self.popen = None
    
    def launch (self):
        try:
            path = os.path.join (config.BIN_DIR, 'tempusfugit')
            self.popen = subprocess.Popen ([path], stderr=subprocess.PIPE)
            
            stderr = self.popen.communicate () [1]
            
            if (stderr is not None) and (stderr != ""):
                self.parent.error_cb ("TempusFugit error", stderr)
        except (OSError, ValueError), e:
            self.parent.error_cb ("Error", \
                    "Cannot execute TempusFugit: \n %s" % str(e))
        except:
            self.parent.error_cb ("Error", \
                    "Cannot execute TempusFugit: \n Unexpected error")
        finally:
            self.parent.reset_launcher ()

