#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2010  Edmundo Álvarez Jiménez
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Edmundo Álvarez Jiménez <e.alvarezj@gmail.com>

import gtk
import os
import config
import gobject
import event
from Squealer import defs

class EventStore (gtk.ListStore):
    """Represents the list of events registered"""
    
    def __init__ (self):
        gtk.ListStore.__init__ (self, gobject.TYPE_STRING, \
                gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
    
    def load (self, events):
        for i in events:
            self.append ([i.get_initial_time (), i.get_length (), \
                    i.get_app (), i.get_title ()])


class OverviewDialog:
    def __init__ (self):
        self.builder = gtk.Builder ()
        self.builder.add_from_file (os.path.join (config.DATA_DIR, \
                "detras-applet", "overview.glade"))
        
        self.window = self.__get_widget ("overview")
        self.window.set_destroy_with_parent (True)
        
        self.builder.connect_signals (self)
        
        self.events_parser = event.EventsParser ()
        events = self.events_parser.parse ()
        
        # Load events
        self.events_treev = self.__get_widget ("tv_events")
        
        self.event_store = EventStore ()
        self.event_store.load (events)
        
        self.date_col = gtk.TreeViewColumn ("Date")
        self.length_col = gtk.TreeViewColumn ("Length")
        self.app_col = gtk.TreeViewColumn ("Application")
        self.title_col = gtk.TreeViewColumn ("Title")
        
        cell = gtk.CellRendererText ()
        cell.set_property ("editable", False)
        
        self.date_col.pack_start (cell, True)
        self.date_col.set_attributes (cell, text = 0)
        self.date_col.set_resizable (True)
        self.date_col.set_sort_column_id (0)
        
        self.length_col.pack_start (cell, True)
        self.length_col.set_attributes (cell, text = 1)
        self.length_col.set_resizable (True)
        self.length_col.set_sort_column_id (1)
        
        self.app_col.pack_start (cell, True)
        self.app_col.set_attributes (cell, text = 2)
        self.app_col.set_resizable (True)
        self.app_col.set_sort_column_id (2)
        
        self.title_col.pack_start (cell, True)
        self.title_col.set_attributes (cell, text = 3)
        self.title_col.set_resizable (True)
        self.title_col.set_sort_column_id (3)

        self.events_treev.append_column (self.date_col)
        self.events_treev.append_column (self.length_col)
        self.events_treev.append_column (self.app_col)
        self.events_treev.append_column (self.title_col)
        
        self.events_treev.set_model (self.event_store)
        
        
    def __get_widget (self, name):
        return self.builder.get_object (name)
    
    def show (self):
        self.window.show_all ()
    
    def get_window (self):
        return self.window
    
    def is_opened (self):
        return self.window is not None
    
    def on_close_button_clicked (self, widget):
        self.window.destroy ()
    
    def on_overview_destroy (self, widget, data=None):
        self.window = None
    
