#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2010  Edmundo Álvarez Jiménez
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Edmundo Álvarez Jiménez <e.alvarezj@gmail.com>

import gtk
import os
import config
import gobject
from Squealer import defs
from Squealer.settings import SettingsWritter

class ProjectStore (gtk.ListStore):
    """Represents the list of projects asociated with the client"""
    
    def __init__ (self):
        gtk.ListStore.__init__ (self, gobject.TYPE_STRING)
    
    def load (self, projects):
        for i in projects:
            self.append ([i])
        

class RegexpStore (gtk.ListStore):
    """Represents the list of regular expressions"""
    
    def __init__ (self):
        gtk.ListStore.__init__ (self, gobject.TYPE_STRING)
    
    def load (self, regexps):
        for i in regexps:
            self.append ([i])
    

class PrefsDialog:
    
    # Columns of stores that are going to be present on tree viewers
    PS_MAIN_COLUMN = 0
    RS_MAIN_COLUMN = 0
    
    def __init__ (self):
        self.builder = gtk.Builder ()
        self.builder.add_from_file (os.path.join (config.DATA_DIR, \
                "detras-applet", "preferences.glade"))
        
        self.window = self.__get_widget ("preferences")
        
        self.window.set_destroy_with_parent (True)
        
        self.builder.connect_signals (self)
        
        self.settings = SettingsWritter ()
        
        # Load initial settings
        start_init = self.settings.get_start_init ()
        self.__get_widget ("cb_start_init").set_active (start_init)
        
        stop_exit = self.settings.get_stop_exit ()
        self.__get_widget ("cb_stop_exit").set_active (stop_exit)
        
        name = self.settings.get_client_name ()
        self.__get_widget ("e_client_name").set_text (name)
        
        upload_name = self.settings.get_send_client_name ()
        self.__get_widget ("cb_send_name").set_active (upload_name)
        
        filter_type = self.settings.get_filter_type ()
        self.__get_widget ("cb_filter").set_active (filter_type is not \
                defs.FILTER_TYPE_MAP ['none'])

        if filter_type is not defs.FILTER_TYPE_MAP ['none']:
            self.__get_widget ("rb_filter_regexp").set_active (filter_type is \
                    defs.FILTER_TYPE_MAP ['regexp'])
            self.__get_widget ("rb_filter_titles").set_active (filter_type is \
                    defs.FILTER_TYPE_MAP ['windows_titles'])
        
        s_address = self.settings.get_server_address ()
        self.__get_widget ("e_address").set_text (s_address)
        
        s_port = self.settings.get_server_port ()
        self.__get_widget ("sb_port").set_value (s_port)
        
        s_sname = self.settings.get_service_name ()
        self.__get_widget ("e_service_name").set_text (s_sname)
        
        # Load saved projects on tv_projects
        self.project_treev = self.__get_widget ("tv_projects")
        
        self.projects = self.settings.get_client_projects ()
        
        self.project_store = ProjectStore ()
        self.project_store.load (self.projects)
        
        self.project_col = gtk.TreeViewColumn ("Project")
        self.project_col.set_expand (True)
        
        cell = gtk.CellRendererText ()
        cell.set_property ("editable", True)
        cell.connect ("edited", self.on_project_edited, \
                self.project_store)
        
        self.project_col.pack_start (cell, True)
        self.project_col.set_attributes (cell, \
                text = PrefsDialog.PS_MAIN_COLUMN)
        
        self.project_treev.append_column (self.project_col)
        self.project_treev.set_model (self.project_store)
        
        self.project_tree_sel = self.project_treev.get_selection ()
        
        # Load saved regexps on tv_regexps
        self.regexp_treev = self.__get_widget ("tv_regexps")
        
        self.regexps = self.settings.get_re_filter_patterns ()
        
        self.regexp_store = RegexpStore ()
        self.regexp_store.load (self.regexps)
        
        self.regexp_col = gtk.TreeViewColumn ("Regexp")
        self.regexp_col.set_expand (True)
        
        cell = gtk.CellRendererText ()
        cell.set_property ("editable", True)
        cell.connect ("edited", self.on_regexp_edited, \
                self.regexp_store)
        
        self.regexp_col.pack_start (cell, True)
        self.regexp_col.set_attributes (cell, \
                text = PrefsDialog.RS_MAIN_COLUMN)
        
        self.regexp_treev.append_column (self.regexp_col)
        self.regexp_treev.set_model (self.regexp_store)
        
        self.regexp_tree_sel = self.regexp_treev.get_selection ()
    
    def __get_widget (self, name):
        return self.builder.get_object (name)
    
    def show (self):
        self.window.show_all ()
    
    def get_window (self):
        return self.window
    
    def is_opened (self):
        return self.window is not None
    
    # Signal handlers on general tab
    def on_cb_start_init_toggled (self, checkbox):
        self.settings.set_start_init (checkbox.get_active ())
    
    def on_cb_stop_exit_toggled (self, checkbox):
        self.settings.set_stop_exit (checkbox.get_active ())
    
    def on_cb_upload_data_toggled (self, checkbox):
        upload_freq = self.__get_widget ("sb_upload_freq")
        freq_type = self.__get_widget ("sb_upload_freq")
        
        if (upload_freq is not None) and (freq_type is not None):
            upload_freq.set_sensitive (checkbox.get_active ())
            freq_type.set_sensitive (checkbox.get_active ())
    
    
    # Signal handlers on client tab
    def on_e_client_name_changed (self, widget):
        self.settings.set_client_name (widget.get_text ())
    
    def on_project_edited (self, cell_renderer, path, new_text, model):
        model [path] [PrefsDialog.PS_MAIN_COLUMN] = new_text
        self.projects [int(path)] = new_text
        self.settings.set_client_projects (self.projects)
    
    def on_b_add_project_clicked (self, button):
        new_cell_iter = self.project_store.append ()
        
        (model, iter) = self.project_tree_sel.get_selected()
        
        new_cell_path = model.get_string_from_iter (new_cell_iter)
        
        self.projects.insert (int(new_cell_path), '')
        self.settings.set_client_projects (self.projects)
        
        self.project_treev.set_cursor_on_cell (new_cell_path, \
                self.project_col, None, True)
    
    def on_b_delete_project_clicked (self, button):
        (model, iter) = self.project_tree_sel.get_selected ()
        if iter is not None:
            path = self.project_store.get_string_from_iter (iter)
            del self.projects [int(path)]
            self.settings.set_client_projects (self.projects)
            
            self.project_store.remove (iter)
    
    
    # Signal handlers on privacy tab
    def on_cb_send_name_toggled (self, checkbox):
        self.settings.set_send_client_name (checkbox.get_active ())
    
    def on_cb_filter_toggled (self, checkbox):
        filter_config = self.__get_widget ("align_filter_config")
        
        # Toggle visibility of filtering options
        if filter_config is not None:
            filter_config.set_sensitive (checkbox.get_active ())
            
            filter_regexp = self.__get_widget ("rb_filter_regexp")
            filter_titles = self.__get_widget ("rb_filter_titles")
           
            if checkbox.get_active ():
                if (filter_regexp is not None) and \
                        (filter_regexp.get_active ()):
                    filter_regexp.toggled ()
                elif filter_titles is not None:
                    filter_titles.toggled ()
        
        # Set filter status
        if not checkbox.get_active ():
            self.settings.set_filter_type (defs.FILTER_TYPE_MAP ['none'])
        
    
    def on_rb_filter_regexp_toggled (self, checkbox):
        regexp_mgr = self.__get_widget ("align_regexp_mgr")
        
        if regexp_mgr is not None:
            regexp_mgr.set_sensitive (checkbox.get_active ())
        
        if checkbox.get_active ():
            self.settings.set_filter_type (defs.FILTER_TYPE_MAP ['regexp'])
    
    def on_rb_filter_titles_toggled (self, checkbox):
        if checkbox.get_active ():
            self.settings.set_filter_type (\
                    defs.FILTER_TYPE_MAP ['windows_titles'])
    
    def on_b_add_regexp_clicked (self, button):
        new_cell_iter = self.regexp_store.append ()
        
        (model, iter) = self.regexp_tree_sel.get_selected()
        
        new_cell_path = model.get_string_from_iter (new_cell_iter)
        
        self.regexps.insert (int(new_cell_path), '')
        self.settings.set_re_filter_patterns (self.regexps)
        
        self.regexp_treev.set_cursor_on_cell (new_cell_path, \
                self.regexp_col, None, True)
    
    def on_b_delete_regexp_clicked (self, button):
        (model, iter) = self.regexp_tree_sel.get_selected ()
        if iter is not None:
            path = self.regexp_store.get_string_from_iter (iter)
            del self.regexps [int(path)]
            self.settings.set_re_filter_patterns (self.regexps)
            
            self.regexp_store.remove (iter)
    
    def on_regexp_edited (self, cell_renderer, path, new_text, model):
        model [path][PrefsDialog.RS_MAIN_COLUMN] = new_text
        self.regexps [int(path)] = new_text
        self.settings.set_re_filter_patterns (self.regexps)
    
    
    # Server signal handlers
    def on_e_address_changed (self, widget):
        self.settings.set_server_address (widget.get_text ())
    
    def on_sb_port_output (self, widget):
        self.settings.set_server_port (widget.get_value_as_int ())
    
    def on_e_service_name_changed (self, widget):
        self.settings.set_service_name (widget.get_text ())
    
    
    # More signal handlers
    def on_close_button_clicked (self, widget):
        self.window.destroy ()
    
    def on_preferences_destroy (self, widget, data=None):
        self.settings.finished ()
        self.window = None
    
