#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2010  Edmundo Álvarez Jiménez
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Edmundo Álvarez Jiménez <e.alvarezj@gmail.com>

import os
import signal
import threading
import config
from launchers import TempusFugitLauncher

class TempusFugitMgr:
    """High level Class to manage TempusFugit execution"""
    
    # Values to indicate TempusFugit's status
    (STOPPED, RUNNING) = range (2)
    
    def __init__ (self, parent):
        self.parent = parent
        username = os.getenv ('LOGNAME')
        self.pid_file = '/tmp/.detras/tempusfugit-%s.pid' % username
        self.tf_launcher = None
        self.tf_lock = threading.Lock ()
    
    def reset_launcher (self):
        self.tf_lock.acquire ()
        self.tf_launcher = None
        self.tf_lock.release ()
    
    def error_cb (self, title, message):
        self.parent.error_cb (title, message)
    
    def launch (self):
        """Start TempusFugit's execution"""
        if self.get_status () is TempusFugitMgr.STOPPED:
            self.tf_lock.acquire ()
            
            if self.tf_launcher is None:
                self.tf_launcher = TempusFugitLauncher (self)
                self.tf_launcher.start ()
            
            self.tf_lock.release ()
    
    def stop (self):
        """Stop TempusFugit's execution"""
        if self.get_status () is TempusFugitMgr.RUNNING:
            file = open (self.pid_file, "r")
            pid = file.read ()
            file.close ()
            
            if (pid is not None) or (pid != ""):
                os.kill (int(pid), signal.SIGTERM)
    
    def get_status (self):
        """Check whether TempusFugit is running or not"""
        
        if os.access (self.pid_file, os.F_OK):
            status = TempusFugitMgr.RUNNING
        else:
            status = TempusFugitMgr.STOPPED
        
        return status

