from pygooglechart import GroupedHorizontalBarChart
from pygooglechart import SimpleLineChart
from pygooglechart import Axis
from pygooglechart import TextData
import math

class ChartGenerator:
    HEIGHT_ADJUST = 155
    HIST_THRESHOLD = 80
    STEPS = {'5': 5, '10': 10, '25': 25, '50': 50, '100': 100}
    
    @staticmethod
    def calculate_height(numres):
        if numres == 1:
            numres += 1
        
        return int(math.log(numres) * ChartGenerator.HEIGHT_ADJUST)
    
    @staticmethod
    def top_applications_by_time(sdata, numres, idate, fdate, project):
        if ((sdata is None) or (sdata == [])):
            return None
        
        numelems = len(sdata)
        
        if numelems < numres:
            numres = numelems
        
        max_x = range(0, int(sdata[0][1]) + ChartGenerator.STEPS['50'] + 1, \
                ChartGenerator.STEPS['50'])[-1]
        
        if project != '%':
            title = 'Top %s applications on %s by usage time (%s - %s)' % \
                    (numres, project, idate.date(), fdate.date())
        else:
            title = 'Top %s applications by usage time (%s - %s)' % \
                    (numres, idate.date(), fdate.date())
        
        chart = GroupedHorizontalBarChart(600, \
                ChartGenerator.calculate_height(numres), \
                title=title, x_range=[0, max_x])
        
        chart.set_colours(['FFCC33'])
        
        xdata = []
        ydata = []
        
        for i in sdata:
            ydata.insert(0, i[0])
            xdata.append(i[1])
        
        chart.add_data(xdata)
        
        bottom_axis = range(0, max_x + 1, ChartGenerator.STEPS['50'])
        chart.set_axis_labels(Axis.BOTTOM, bottom_axis)
        
        chart.set_axis_labels(Axis.LEFT, ydata)
        
        chart.set_grid((100*ChartGenerator.STEPS['25'])/float(max_x), 0, 2, 5)
        
        if numres > 10:
            chart.set_bar_width(10)
        
        return chart.get_url(data_class=TextData)
    
    @staticmethod
    def top_applications_by_users(sdata, numres, idate, fdate, project):
        if ((sdata is None) or (sdata == [])):
            return None
        
        numelems = len(sdata)
        
        if numelems < numres:
            numres = numelems
        
        max_x = range(0, int(sdata[0][1]) + ChartGenerator.STEPS['25'] + 1, \
                ChartGenerator.STEPS['25'])[-1]

        if project != '%':
            title = 'Top %s applications on %s by number of users (%s - %s)' % \
                    (numres, project, idate.date(), fdate.date())
        else:
            title = 'Top %s applications by number of users (%s - %s)' % \
                    (numres, idate.date(), fdate.date())
        
        chart = GroupedHorizontalBarChart(600, \
                ChartGenerator.calculate_height(numres), \
                title=title, x_range=[0, max_x])
        
        chart.set_colours(['80C65A'])
        
        xdata = []
        ydata = []
        
        for i in sdata:
            ydata.insert(0, i[0])
            xdata.append(i[1])
        
        chart.add_data(xdata)
        
        bottom_axis = range(0, max_x + 1, ChartGenerator.STEPS['25'])
        chart.set_axis_labels(Axis.BOTTOM, bottom_axis)
        
        chart.set_axis_labels(Axis.LEFT, ydata)
        
        chart.set_grid((100*ChartGenerator.STEPS['25'])/float(max_x), 0, 2, 5)
        
        if numres > 10:
            chart.set_bar_width(10)
        
        return chart.get_url(data_class=TextData)
    
    @staticmethod
    def most_active_clients(sdata, numres, idate, fdate, project):
        if ((sdata is None) or (sdata == [])):
            return None
        
        numelems = len(sdata)
        
        if numelems < numres:
            numres = numelems
        
        max_x = range(0, int(sdata[0][1]) + ChartGenerator.STEPS['100'] + 1, \
                ChartGenerator.STEPS['100'])[-1]
        
        if project != '%':
            title = '%s most active clients on %s (%s - %s)' % \
                    (numres, project, idate.date(), fdate.date())
        else:
            title = '%s most active clients (%s - %s)' % \
                    (numres, idate.date(), fdate.date())
        
        chart = GroupedHorizontalBarChart(600, \
                ChartGenerator.calculate_height(numres), \
                title=title, x_range=[0, max_x])
        
        chart.set_colours(['76A4FB'])
        
        xdata = []
        ydata = []
        
        for i in sdata:
            ydata.insert(0, i[0])
            xdata.append(i[1])
        
        chart.add_data(xdata)
        
        bottom_axis = range(0, max_x + 1, ChartGenerator.STEPS['100'])
        chart.set_axis_labels(Axis.BOTTOM, bottom_axis)
        
        chart.set_axis_labels(Axis.LEFT, ydata)
        
        chart.set_grid((100*ChartGenerator.STEPS['50'])/float(max_x), 0, 2, 5)
        
        if numres > 10:
            chart.set_bar_width(10)
        
        return chart.get_url(data_class=TextData)
    
    @staticmethod
    def top_projects_by_time(sdata, numres, idate, fdate):
        if ((sdata is None) or (sdata == [])):
            return None
        
        numelems = len(sdata)
        
        if numelems < numres:
            numres = numelems
        
        max_x = range(0, int(sdata[0][1]) + ChartGenerator.STEPS['50'] + 1, \
                ChartGenerator.STEPS['50'])[-1]
        
        chart = GroupedHorizontalBarChart(600, \
                ChartGenerator.calculate_height(numres), \
                title='%s most active projects by tracked time (%s - %s)' \
                % (numres, idate.date(), fdate.date()), \
                x_range=[0, max_x])
        
        chart.set_colours(['990066'])
        
        xdata = []
        ydata = []
        
        for i in sdata:
            ydata.insert(0, i[0])
            xdata.append(i[1])
        
        chart.add_data(xdata)
        
        bottom_axis = range(0, max_x + 1, ChartGenerator.STEPS['50'])
        chart.set_axis_labels(Axis.BOTTOM, bottom_axis)
        
        chart.set_axis_labels(Axis.LEFT, ydata)
        
        chart.set_grid((100*ChartGenerator.STEPS['25'])/float(max_x), 0, 2, 5)
        
        if numres > 10:
            chart.set_bar_width(10)
        
        return chart.get_url(data_class=TextData)
    
    @staticmethod
    def top_projects_by_members(sdata, numres):
        if ((sdata is None) or (sdata == [])):
            return None
        
        numelems = len(sdata)
        
        if numelems < numres:
            numres = numelems
        
        max_x = range(0, int(sdata[0][1]) + ChartGenerator.STEPS['25'] + 1, \
                ChartGenerator.STEPS['25'])[-1]
        
        chart = GroupedHorizontalBarChart(600, \
                ChartGenerator.calculate_height(numres), \
                title='%s most active projects by number of members' % numres, \
                x_range=[0, max_x])
        
        chart.set_colours(['CC0000'])
        
        xdata = []
        ydata = []
        
        for i in sdata:
            ydata.insert(0, i[0])
            xdata.append(i[1])
        
        chart.add_data(xdata)
        
        bottom_axis = range(0, max_x + 1, ChartGenerator.STEPS['25'])
        chart.set_axis_labels(Axis.BOTTOM, bottom_axis)
        
        chart.set_axis_labels(Axis.LEFT, ydata)
        
        chart.set_grid((100*ChartGenerator.STEPS['25'])/float(max_x), 0, 2, 5)
        
        if numres > 10:
            chart.set_bar_width(10)
        
        return chart.get_url(data_class=TextData)

    @staticmethod
    def historical_chart(data, name):
        curr_max = 0
        
        for i in data:
            if i[1] > curr_max:
                curr_max = i[1]
        
        if curr_max < ChartGenerator.HIST_THRESHOLD:
            max_y = range(0, int(curr_max) + ChartGenerator.STEPS['5'] + 1, \
                    ChartGenerator.STEPS['5'])[-1]
        else:
            max_y = range(0, int(curr_max) + ChartGenerator.STEPS['10'] + 1, \
                    ChartGenerator.STEPS['10'])[-1]
        print max_y
        
        chart = SimpleLineChart(600, 500, \
                title='%s 6 months evolution by tracked time' % name, \
                x_range=[0, len(data)-1], y_range=[0, max_y])
        
        chart.set_colours(['0000FF'])
        
        xdata = []
        ydata = []
        
        for i in data:
            xdata.append(i[1])
            ydata.append(i[0])
        
        chart.add_data(xdata)
        
        chart.set_axis_labels(Axis.BOTTOM, ydata)
        chart.set_axis_labels(Axis.BOTTOM, ['', 'Months', ''])
        
        if max_y < ChartGenerator.HIST_THRESHOLD:
            left_axis = range(0, max_y+1, ChartGenerator.STEPS['5'])
        else:
            left_axis = range(0, max_y+1, ChartGenerator.STEPS['10'])
        chart.set_axis_labels(Axis.LEFT, left_axis)
        chart.set_axis_labels(Axis.LEFT, ['', 'Hours', ''])
        
        if max_y < ChartGenerator.HIST_THRESHOLD:
            chart.set_grid((100/float(len(data)-1)), (100/float(max_y)), 2, 5)
        else:
            chart.set_grid((100/float(len(data)-1)), \
                    ((100*10)/float(max_y)), 2, 5)
        
        return chart.get_url(data_class=TextData)

