from dazer.analyzer.models import *
from django.utils.datastructures import MultiValueDictKeyError
from django.db import connection
from django.shortcuts import render_to_response
from dazer.analyzer.resources.WebResource import *
from dazer.analyzer.charts.ChartGenerator import ChartGenerator
from operator import itemgetter
from datetime import datetime
from datetime import timedelta
from time import mktime

SECTOHOURS = float(60*60)

class AppsCharts(WebResource):
    
    def process_GET(self):
        error = None
        name = None
        
        try:
            name = self.request.GET['project']
            project_f = Project.objects.filter(name__iexact=name)
            project_f = project_f[0].name
        except:
            project_f = '%'
            
            if ((name is not None) and (name != '')):
                error = "Project '%s' not found in DeTraS registers" % name
                return render_to_response('applications.html', \
                        {'error': error, })
        
        try:
            fday = self.request.GET['fday']
            fmonth = self.request.GET['fmonth']
            fyear = self.request.GET['fyear']
            
            if (((fday == 'dd') and (fmonth == 'mm') and (fyear == 'yyyy')) \
                    or ((fday == '') and (fmonth == '') and (fyear == ''))):
                fdate = datetime.now()
            else:
                fdate = datetime(int(fyear), int(fmonth), int(fday))
        except MultiValueDictKeyError:
            fdate = datetime.now()
        except:
            fdate = datetime.now()
            error = "Invalid final date, using default one"
        
        try:
            tfdate = int(mktime(fdate.timetuple()))
        except:
            error = "Final date out of range. Please check if it is right"
            return render_to_response('applications.html', {'error': error, })
        
        try:
            iday = self.request.GET['iday']
            imonth = self.request.GET['imonth']
            iyear = self.request.GET['iyear']
            
            if (((iday == 'dd') and (imonth == 'mm') and (iyear == 'yyyy')) \
                    or ((iday == '') and (imonth == '') and (iyear == ''))):
                idate = fdate - timedelta(days=30)
            else:
                idate = datetime(int(iyear), int(imonth), int(iday))
        except MultiValueDictKeyError:
            idate = fdate - timedelta(days=30)
        except:
            idate = fdate - timedelta(days=30)
            error = "Invalid initial date, using default one"
         
        try: 
            tidate = int(mktime(idate.timetuple()))
        except:
            error = "Initial date out of range. Please check if it is right"
            return render_to_response('applications.html', {'error': error, })
        
        if idate >= fdate:
            error = 'Initial date must be later than final date'
            return render_to_response('applications.html', {'error': error, })
        
        try:
            numres = int(self.request.GET['numres'])
            
            if numres > 25:
                numres = 25
            
            if numres < 3:
                numres = 3
        except:
            numres = 5
        
        if self.an_type == 'time':
            win_etype = EventType.objects.get(name__iexact='window')
            cursor = connection.cursor()
            data = {}
            
            cursor.execute( \
                    "select ew.application, sum(e.duration) from events " \
                    + "e, eventwindow ew, clients c, projects p where " \
                    + "e.event_type = %s and ew.id = e.event and " \
                    + "e.duration is not null and e.duration <> 0 and " \
                    + "e.timestamp >= %s and e.timestamp <= %s and " \
                    + "c.id = p.client_id and e.client_id = c.id and " \
                    + "p.name like %s group by ew.application", \
                    [win_etype.id, tidate, tfdate, project_f])
            
            for i in cursor.fetchall():
                app = i[0]
                duration = float(i[1])
                
                data[app] = duration / SECTOHOURS
            
            sdata = sorted(data.iteritems(), key=itemgetter(1), \
                    reverse=True)[:numres]
            
            chart = ChartGenerator.top_applications_by_time(sdata, numres, \
                    idate, fdate, project_f)
            
            return render_to_response('applications.html', \
                    {'type': self.an_type, 'data': sdata, 'chart': chart, \
                    'error': error, })
        elif self.an_type == 'users':
            win_etype = EventType.objects.get(name__iexact='window')
            cursor = connection.cursor()
            data = {}
            
            cursor.execute( \
                    "select distinct e.client_id, ew.application from " \
                    + "events e, eventwindow ew, clients c, projects p " \
                    + "where e.event_type = %s and ew.id = e.event and " \
                    + "e.timestamp >= %s and e.timestamp <= %s and " \
                    + "ew.application <> '(null)' and c.id = p.client_id " \
                    + "and e.client_id = c.id and p.name like %s", \
                    [win_etype.id, tidate, tfdate, project_f])
            
            for i in cursor.fetchall():
                client = i[0]
                app = i[1]
                
                if app in data.keys():
                    data[app] += 1
                else:
                    data[app] = 1
            
            sdata = sorted(data.iteritems(), key=itemgetter(1), \
                    reverse=True)[:numres]
            
            chart = ChartGenerator.top_applications_by_users(sdata, numres, \
                    idate, fdate, project_f)
            
            return render_to_response('applications.html', \
                    {'type': self.an_type, 'data': sdata, 'chart': chart, \
                    'error': error, })
        else:
            return render_to_response('applications.html')

