from dazer.analyzer.models import *
from django.utils.datastructures import MultiValueDictKeyError
from django.db import connection
from django.shortcuts import render_to_response
from dazer.analyzer.resources.WebResource import *
from dazer.analyzer.charts.ChartGenerator import ChartGenerator
from operator import itemgetter
from datetime import datetime
from datetime import timedelta
from time import mktime

SECTOHOURS = float(60*60)

class ProjectsCharts(WebResource):
    
    def process_GET(self):
        try:
            numres = int(self.request.GET['numres'])
            
            if numres > 25:
                numres = 25
            
            if numres < 3:
                numres = 3
        except:
            numres = 5
        
        if self.an_type == 'time':
            error = None
            
            try:
                fday = self.request.GET['fday']
                fmonth = self.request.GET['fmonth']
                fyear = self.request.GET['fyear']
                
                if (((fday == 'dd') and (fmonth == 'mm') and (fyear == 'yyyy')) \
                    or ((fday == '') and (fmonth == '') and (fyear == ''))):
                    fdate = datetime.now()
                else:
                    fdate = datetime(int(fyear), int(fmonth), int(fday))
            except MultiValueDictKeyError:
                fdate = datetime.now()
            except:
                fdate = datetime.now()
                error = "Invalid final date, using default one"
            
            try:
                tfdate = int(mktime(fdate.timetuple()))
            except:
                error = "Final date out of range. Please check if it is right"
                return render_to_response('projects.html', {'error': error, })
            
            try:
                iday = self.request.GET['iday']
                imonth = self.request.GET['imonth']
                iyear = self.request.GET['iyear']
                
                if (((iday == 'dd') and (imonth == 'mm') and (iyear == 'yyyy')) \
                    or ((iday == '') and (imonth == '') and (iyear == ''))):
                    idate = fdate - timedelta(days=30)
                else:
                    idate = datetime(int(iyear), int(imonth), int(iday))
            except MultiValueDictKeyError:
                idate = fdate - timedelta(days=30)
            except:
                idate = fdate - timedelta(days=30)
                error = "Invalid initial date, using default one"
            
            try:
                tidate = int(mktime(idate.timetuple()))
            except:
                error = "Initial date out of range. Please check if it is right"
                return render_to_response('projects.html', {'error': error, })
            
            if idate >= fdate:
                error = 'Initial date must be later than final date'
                return render_to_response('projects.html', \
                        {'error': error,})
            
            win_etype = EventType.objects.get(name__iexact='window')
            cursor = connection.cursor()
            data = {}
            
            cursor.execute( \
                    "select p.name, sum(e.duration) from projects p, events " \
                    + "e where e.event_type = %s and e.client_id = " \
                    + "p.client_id and e.duration is not null and " \
                    + "e.duration <> 0 and e.timestamp >= %s and " \
                    + "e.timestamp <= %s group by p.name", \
                    [win_etype.id, tidate, tfdate])
            
            for i in cursor.fetchall():
                project = i[0]
                duration = float(i[1])
                
                data[project] = duration / SECTOHOURS
            
            sdata = sorted(data.iteritems(), key=itemgetter(1), \
                    reverse=True)[:numres]
            
            chart = ChartGenerator.top_projects_by_time(sdata, numres, idate, \
                    fdate)
            
            return render_to_response('projects.html', \
                    {'type': self.an_type, 'data': sdata, 'chart': chart, \
                    'error': error,})
        elif self.an_type == 'members':
            cursor = connection.cursor()
            data = {}
            
            cursor.execute( \
                    "select distinct name, client_id from projects order " \
                    + "by name")
            
            for i in cursor.fetchall():
                project = i[0]
                client = i[1]
                
                if project in data.keys():
                    data[project] += 1
                else:
                    data[project] = 1
            
            sdata = sorted(data.iteritems(), key=itemgetter(1), \
                    reverse=True)[:numres]
            
            chart = ChartGenerator.top_projects_by_members(sdata, numres)
            
            return render_to_response('projects.html', \
                    {'type': self.an_type, 'data': sdata, 'chart': chart,})
        else:
            return render_to_response('projects.html')

