#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>

import Reports

class GraphFunc:

    def __init__ (self, name, app = None, title = None, step = None, timeval = None):
        self.name = name
        self.app = app
        self.title = title
        self.step = step
        self.timeval = timeval

    def get_name (self):
        return self.name

    def get_application (self):
        return self.app

    def get_title (self):
        return self.title

    def get_step (self):
        return self.step

    def get_timeval (self):
        return self.timeval


class Graphs:

    def __init__ (self, dbfile):
        self.reps = Reports.Reports (dbfile)
        self.output_type = "png"
        self.key_type = "outside below box"

    def __get_func_points (self, func):
        points = []

        app = func.get_application ()
        title = func.get_title ()
        step = func.get_step ()
        timeval = func.get_timeval ()

        intval = (timeval[0], min (timeval[0] + step, timeval[1]))
        next = timeval[0] + 1
        x = 1
        while next <= intval[1]:
            cum = self.reps.get_cumulative_duration (app = app, title = title, timeval = intval)
            points.append ((intval[1], cum))
            x += 1
            next = intval[1] + 1
            intval = (next, min (next + step, timeval[1]))
            
        return points

    def __get_func_duration (self, func):
        app = func.get_application ()
        title = func.get_title ()
        timeval = func.get_timeval ()

        return self.reps.get_cumulative_duration (app = app, title = title, timeval = timeval)

    def set_output_type (self, type):
        self.output_type = type

    def set_key_type (self, type):
        self.key_type = type

    def create_graph_func (self, title, xlabel, ylabel, funcs):
        from datetime import timedelta as td

        plot = []

        plot.append ("set terminal %s" % (self.output_type))
        plot.append ("set pointsize 1")
        plot.append ("set grid")
        plot.append ("set key %s" % (self.key_type))
        plot.append ("set title \"%s\"" % (title))
        plot.append ("set xlabel \"%s\"" % (xlabel))
        plot.append ("set ylabel \"%s\"" % (ylabel))
        plot.append ("set ydata time")
        plot.append ("set xdata time")
        plot.append ("set timefmt \"%s\"")

        command = "plot "
        i = 0
        for func in funcs:
            filename = "%d-%d_%s.points" % (func.get_timeval ()[0], func.get_timeval ()[1], func.get_name ())
            f = open (filename, "w")
            points = self.__get_func_points (func)
            for point in points:
                f.write ("%d  %d\n" % (point[0], point[1]))
                
            f.close ()
            if i > 0:
                command += ","
            command += "\"%s\" using 1:2 t '%s' with lines" % (filename, func.get_name ())
            i += 1
        
        plot.append (command)

        for line in plot:
            print line

    def create_graph_boxes (self, title, funcs):
        from datetime import timedelta as td

        plot = []

        plot.append ("set terminal %s" % (self.output_type))
        plot.append ("set boxwidth 0.5")
        plot.append ("set style fill solid 0.50 border -1")
        plot.append ("set grid")
        plot.append ("unset key")
        plot.append ("set title \"%s\"" % (title))
        
        filename = "graph-data.boxes"
        f = open (filename, "w")

        command = "plot "
        xtics = "set xtics ("
        ytics = "set ytics ("
        
        max_duration = 0
        i = 0
        j = 1
        for func in funcs:
            duration = self.__get_func_duration (func)
            f.write ("%d %d " % (i + 1, duration))

            if duration > max_duration:
                max_duration = duration

            if i > 0:
                xtics += ","
                ytics += ","
                command += ","
            xtics += "'%s' %d" % (func.get_name (), i + 1)
            ytics += "'%s' %d" % (td (seconds = duration), duration)
            command += "\"%s\" using %d:%d with boxes" % (filename, j, j + 1)
            i += 1
            j += 2

        f.close ()
        xtics += ")"
        ytics += ")"
        
        plot.append (xtics)
        plot.append (ytics)

        plot.append ("set xrange[0:%d]" % (len (funcs) + 1))
        plot.append ("set yrange[0:%d]" % (max_duration + 60 * 60))

        plot.append (command)

        for line in plot:
            print line

        
