#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>

import os

import gobject
import gtk
import gtk.glade

import Reports

gui = None

def get_gui ():
    global gui

    if gui is None:
        gui = gtk.glade.XML (os.path.join ('data', 'report.glade'))

    return gui


(
        COLUMN_DATE,
        COLUMN_APP,
        COLUMN_TITLE,
        COLUMN_DURATION
) = range (4)

class ReportList (gobject.GObject):

    def __init__ (self, reports):
        self.__gobject_init__()

        self.reports = reports
        
        self.treeview = get_gui ().get_widget ('list')

        model = self.__create_model ()
        self.treeview.set_model (model)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ('Date', renderer, text = COLUMN_DATE)
        self.treeview.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ('Application', renderer, text = COLUMN_APP)
        self.treeview.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ('Window Title', renderer, text = COLUMN_TITLE)
        self.treeview.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ('Duration', renderer, text = COLUMN_DURATION)
        self.treeview.append_column (column)

        self.treeview.show ()


    def __create_model (self):
        store = gtk.ListStore (
                gobject.TYPE_STRING, 
                gobject.TYPE_STRING,
                gobject.TYPE_STRING,
                gobject.TYPE_STRING)

        acts = self.reports.get_activities ()
        for client in acts:
            # TODO: per client list
            for activity in acts[client]:
                iter = store.append ()
                store.set (iter, 
                        COLUMN_DATE, activity.get_date (), 
                        COLUMN_APP, activity.get_application (), 
                        COLUMN_TITLE, activity.get_title (),
                        COLUMN_DURATION, "%d" % (activity.get_duration ()))

        return store
                

class ReportWindow (gobject.GObject):
    
    def __init__ (self, reports):
        self.__gobject_init__()

        self.win = get_gui ().get_widget ('main_window')
        self.win.connect ('destroy', gtk.main_quit)

        self.list = ReportList (reports)

        self.win.show ()

class ReportGUI:

    def __init__ (self, db):
        self.reports = Reports.Reports (db)
        self.window = ReportWindow (self.reports)

    def run (self):
        gtk.main ()

if __name__ == '__main__':
    repgui = ReportGUI ("./detras.db")
    repgui.run ()
