#!/usr/bin/env python
# Copyright (C) 2006  Carlos Garcia Campos
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>

from threading import Thread
from PyDataAccess.ConnectionFactory import ConnectionFactory as cf

class DataManager (Thread):
    
    def __init__ (self, gconfig):
        Thread.__init__ (self)
        
        self.config = gconfig
 
        self.db = None
        self.cb = None
        self.pkg = None
        
    def save (self, pkg):
        self.pkg = pkg

        return self.start ()

    def add_callback (self, cb):
        self.cb = cb

    def run (self):
        if self.db is None:
            module = self.config.get_db_module ()
            conn = self.config.get_db_connection_string ()

            self.db = cf.create_connector (module)
            self.db.connect (db=conn)

        client_id = self.pkg["metadata"].get_id ()
        self.pkg["metadata"].db_insert (self.db)

        for event in self.pkg["events"]:
            time = event.get_time ()
            duration = event.get_duration ()
            event_type = event.db_get_type (self.db)
            event_id = event.db_get_id (self.db)
            
            command = "insert into Events (client_id, timestamp, event_type, event, duration) "
            command += "values (?, ?, ?, ?, ?)"
            self.db.execute (command, (client_id, time, event_type, event_id, duration))

        if self.cb is not None:
            self.cb ()
