#!/usr/bin/env python
# Copyright (C) 2006  Carlos Garcia Campos
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>

import EventDB

class Event (EventDB.EventDB):

    def __init__ (self, time, name):
        self.name = name
        self.time = time
        self.duration = None

    def get_time (self):
        return self.time

    def set_duration (self, duration):
        self.duration = duration

    def get_duration (self):
        return self.duration

    # EventDB interface
    def db_get_type (self, db):
        command = "select id from EventTypes where name = ?"
        try:
            id = db.execute (command, (self.name,))[0][0]
        except:
            command = "insert into EventTypes (name) values (?)"
            db.execute (command, (self.name,))

            return self.db_get_type (db)

        return id

class Window (Event):

    def __init__ (self, time):
        Event.__init__ (self, time, "Window")

        self.app = "unknown"
        self.title = "unknown"

    def set_app (self, app):
        self.app = app

    def set_title (self, title):
        self.title = title

    # EventDB interface
    def db_get_id (self, db):
        command = "select id from EventWindow where application = ? and title = ?"
        try:
            id = db.execute (command, (self.app, self.title))[0][0]
        except:
            command = "insert into EventWindow (application, title) values (?, ?)"
            db.execute (command, (self.app, self.title))

            return self.db_get_id (db)

        return id

class Idle (Event):

    def __init__ (self, time):
        Event.__init__ (self, time, "Idle")

    # EventDB interface
    def db_get_id (self, db):
        return None

class Session (Event):

    def __init__ (self, time, type):
        Event.__init__ (self, time, "Session")
        self.type = type

    # EventDB interface
    def db_get_id (self, db):
        command = "select id from EventSession where name = ?"
        try:
            id = db.execute (command, (self.type,))[0][0]
        except:
            command = "insert into EventSession (name) values (?)"
            db.execute (command, (self.type,))

            return self.db_get_id (db)

        return id

