#!/usr/bin/env python
# Copyright (C) 2006  Carlos Garcia Campos
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>

from threading import Thread
import xml.sax.handler

import Event
import Client

class EventsHandler (xml.sax.handler.ContentHandler):

    def __init__ (self):
        self.events = []
        self.event = None
        self.current = None
        self.buffer = ""

    def startElement (self, name, attributes):
        if name == 'event':
            type = attributes['type']
            time = attributes['time']

            if len (self.events) > 0:
                previous = self.events[-1]
                previous.set_duration (int (time) - int (previous.get_time ()))
            
            if type == 'WindowChange':
                self.event = Event.Window (time)
            elif type == 'InactiveTime':
                self.event = Event.Idle (time)
            elif type == 'Session':
                ev = attributes['event']
                self.event = Event.Session (time, ev)
        else:
            self.current = name
                
    def characters (self, data):
        if self.current is None:
            return
        if self.event is None:
            return

        if self.current == 'app' or self.current == 'title':
            self.buffer += data

    def endElement (self, name):
        self.current = None
        
        if name == 'event':
            self.events.append (self.event)
            self.event = None
        elif name == 'app' and self.buffer != "":
            self.event.set_app (self.buffer)
            self.buffer = ""
        elif name == 'title' and self.buffer != "":
            self.event.set_title (self.buffer)
            self.buffer = ""

    def getEvents (self):
        return self.events
            
class MetadataHandler (xml.sax.handler.ContentHandler):

    def __init__ (self):
        self.metadata = None
        self.client = None
        self.current = None
        self.buffer = ""

    def startElement (self, name, attributes):
        if name == 'client':
            id = attributes['id']
            self.client = Client.Client (id)
        else:
            self.current = name
                
    def characters (self, data):
        if self.current is None:
            return
        if self.client is None:
            return

        if self.current == 'name' or self.current == 'project':
            self.buffer += data

    def endElement (self, name):
        self.current = None
        
        if name == 'client':
            self.metadata = self.client
            self.client = None
        elif name == 'name' and self.buffer != "":
            self.client.set_name (self.buffer)
            self.buffer = ""
        elif name == 'project' and self.buffer != "":
            self.client.add_project (self.buffer)
            self.buffer = ""

    def getMetadata (self):
        return self.metadata


class Register (Thread):

    config = None
    
    def __init__ (self, gconfig):
        Thread.__init__ (self)

        self.cb = None
        
        if Register.config is None:
            Register.config = gconfig

    def register (self, header, body):
        self.header = header.encode ('UTF-8')
        self.body = body.encode ('UTF-8')

        return self.start ()

    def add_callback (self, cb):
        self.cb = cb

    def run (self):
        pkg = {}
        
        handler = EventsHandler ()
       
        parser = xml.sax.make_parser ()
        parser.setContentHandler (handler)
        parser.feed (self.body)
        
        events = handler.getEvents ()
        pkg["events"] = events

        handler = MetadataHandler ()

        parser = xml.sax.make_parser ()
        parser.setContentHandler (handler)
        parser.feed (self.header)

        metadata = handler.getMetadata ()
        pkg["metadata"] = metadata

        if self.cb is not None:
            self.cb (pkg)


