#!/usr/bin/env python
# Copyright (C) 2006  Carlos Garcia Campos
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>

from twisted.web import server, resource, soap
import thread
import Register
import DataManager

class Service (soap.SOAPPublisher):

    TIMEOUT = 1
    
    def __init__ (self, gconfig):
        from twisted.internet import reactor
        
        self.pkgs_lock = thread.allocate_lock ()
        self.saver_lock = thread.allocate_lock ()
        
        self.config = gconfig

        self.packages = []

        self.reactor = reactor
        self.reactor.callLater (Service.TIMEOUT, self._check_and_save)

    def soap_send (self, header, body):
        register = Register.Register (self.config)
        register.add_callback (self.register_finished)
        register.register (header, body)

    def register_finished (self, pkg):
        self.pkgs_lock.acquire ()
        self.packages.insert (0, pkg)
        self.pkgs_lock.release ()

    def _check_and_save (self):
        if len (self.packages) > 0:
            if self.saver_lock.acquire (False):
                self.pkgs_lock.acquire ()
                pkg = self.packages.pop ()
                self.pkgs_lock.release ()
               
                dm = DataManager.DataManager (self.config)
                dm.add_callback (self.save_finished)
                dm.save (pkg)

        self.reactor.callLater (Service.TIMEOUT, self._check_and_save)

    def save_finished (self):
        self.saver_lock.release ()

from twisted.application import service, internet

class ApplicationService (internet.SSLServer):
    
    def __init__ (self, gconfig):
        from twisted.internet import ssl
        
        service_uri = resource.Resource ()
        service_uri.putChild ('DeTraS', Service (gconfig))
        
        ssl_context = ssl.DefaultOpenSSLContextFactory (\
                gconfig.get_ssl_private_key (), \
                gconfig.get_ssl_certificate ())
        port = gconfig.get_server_port ()
        
        internet.SSLServer.__init__ (self, port, server.Site (service_uri), ssl_context)

