#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2006  Carlos Garcia Campos
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>

import os
import time

class FileLocker:

    def __init__ (self, timeout = 120):
        self.timeout = timeout
    
    def lock (self, file):
        if not os.path.exists (file):
            return
        
        lfile = file + ".lock"
        
        secs = 0
        while True:
            try:
                os.link (file, lfile)
                break
            except:
                time.sleep (1)
                secs += 1

                if secs == self.timeout:
                    try:
                        os.unlink (lfile)
                    except:
                        pass
        

    def unlock (self, file):
        if not os.path.exists (file):
            return

        lfile = file + ".lock"

        try:
            os.unlink (lfile)
        except:
            pass

if __name__ == "__main__":
    locker = FileLocker (60)

    print "Getting lock"
    locker.lock ("/home/carlos/.detras/tempusfugit.xml")
    print "locked"

    time.sleep (20)

    print "unlocking"
    locker.unlock ("/home/carlos/.detras/tempusfugit.xml")
    print "unlocked"

