#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2006  Carlos Garcia Campos
# Copyright (C) 2010  Edmundo Álvarez Jiménez
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
#          Edmundo Álvarez Jiménez <e.alvarezj@gmail.com>

from xml.dom import minidom
import re

class TypeError (Exception):
    """
    Represents an error occurred when filter type (represented by an int)
    is not in defs.FILTER_TYPE_MAP.
    """
    
    def __init__ (self, msg):
        self.msg = msg
    
    def __repr__ (self):
        return self.msg
    
    def __str__ (self):
        return self.msg


class Filter:
    def __init__ (self, gconfig):
        self.config = gconfig
        self.xml_file = self.config.get_events_file ()
    
    def read_xml (self):
        f = open (self.xml_file, "r")
        xml = f.read ()
        f.close ()
        
        return xml


class NoFilter (Filter):
    """Do not filter anything"""
    def __init__ (self, gconfig):
        Filter.__init__ (self, gconfig)
    
    def filter (self):
        return Filter.read_xml (self)


class RegExpFilter (Filter):
    """Filter windows titles using a regular expression"""
    def __init__ (self, gconfig):
        Filter.__init__ (self, gconfig)
    
    def filter (self):
        xml = Filter.read_xml (self)
        xml_doc = minidom.parseString (xml)
        
        titles = xml_doc.getElementsByTagName ("title")
        
        patterns = self.config.get_re_filter_patterns ()
        
        for i in titles:
            titleNode = i.firstChild
            
            if titleNode is not None:
                for j in patterns:
                    titleNode.data = re.subn (j, '', titleNode.data) [0]
        
        doc = xml_doc.toxml ("utf-8")
        xml_doc.unlink ()
        
        return doc


class WindowsTitlesFilter (Filter):
    """Filter windows titles"""
    def __init__ (self, gconfig):
        Filter.__init__ (self, gconfig)
    
    def filter (self):
        xml = Filter.read_xml (self)
        xml_doc = minidom.parseString (xml)
        
        titles = xml_doc.getElementsByTagName ("title")
        
        for i in titles:
            parent = i.parentNode
            parent.removeChild (i)
        
        doc = xml_doc.toxml ("utf-8")
        xml_doc.unlink ()
        
        return doc


