#!/usr/bin/env python
# coding=utf8
#
# Copyright (C) 2006  Carlos Garcia Campos
# Copyright (C) 2010  Edmundo Álvarez Jiménez
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Authors: Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
#          Edmundo Álvarez Jiménez <e.alvarezj@gmail.com>

import sys

def print_error ():
    import traceback
    
    eType, eValue, eTraceback = sys.exc_info()
    error = traceback.format_exception_only (eType, eValue) [0]
    print >> sys.stderr, "%s" % error

def main (args):
    import Filter
    import Sender
    import Register
    import Client
    import FileLocker
    import defs
    from settings import SettingsReader
    
    # File Locker
    locker = FileLocker.FileLocker ()
    
    # Global configuration
    gconfig = SettingsReader ()
    
    events_file = gconfig.get_events_file ()
    
    locker.lock (events_file)
    
    filter_type = gconfig.get_filter_type ()
    
    # Filter events file
    try:
        if filter_type == defs.FILTER_TYPE_MAP ['none']:
            filter = Filter.NoFilter (gconfig)
        elif filter_type == defs.FILTER_TYPE_MAP ['regexp']:
            filter = Filter.RegExpFilter (gconfig)
        elif filter_type == defs.FILTER_TYPE_MAP ['windows_titles']:
            filter = Filter.WindowsTitlesFilter (gconfig)
        else:
            raise Filter.TypeError ("%d is not a valid filter type" \
                    % filter_type)
   
        xml_filtered = filter.filter ()
    except IOError:
        exit (0)
    except:
        locker.unlock (events_file)
        print >> sys.stderr, "Error found while filtering"
        print_error ()
        exit (1)
    
    # Get client info
    metadata = Client.Client (gconfig)
    try:
        xml_header = metadata.get_xml ()
    except:
        locker.unlock (events_file)
        print >> sys.stderr, "Error found while generating client info"
        print_error ()
        exit (1)
    
    # Send trace file filtered to remote server
    sender = Sender.Sender (gconfig)
    try:
        sender.send (xml_header, xml_filtered)
    except:
        locker.unlock (events_file)
        print >> sys.stderr, "Error found while sending data"
        print_error ()
        exit (1)
    
    # Save sent data
    register = Register.Register (gconfig)
    try:
        register.register ()
    except:
        locker.unlock (events_file)
        print >> sys.stderr, "Error found while storing data"
        print_error ()
        exit (1)
    
    locker.unlock (events_file)

if __name__ == "__main__":
    main (None)
