/*
 *  Copyright (C) 2006 Carlos Garcia Campos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Authors : Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
 */

#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "tf-daemon.h"

#define TF_LAUNCHER BINDIR"/tf-launch"

static char *
tf_daemon_get_username (void)
{
	struct passwd *pw = NULL;
	
	setpwent ();
	pw = getpwuid (getuid ());
	endpwent ();

	if (!pw)
		return NULL;

	return strdup (pw->pw_name);
}

static const char *
tf_daemon_get_filename (void)
{
	char         *username;
	unsigned int  len;
	static char  *filename = NULL;

	if (filename)
		return filename;

	username = tf_daemon_get_username ();

	len = strlen (username) + strlen ("/tmp/.detras/tempusfugit-.pid") + 1;
	filename = (char *) malloc (sizeof (char) * len);

	snprintf (filename, len, "/tmp/.detras/tempusfugit-%s.pid", username);
	free (username);

	return filename;
}

TfDaemonStatus
tf_daemon_get_status (void)
{
	const char    *filename;
	TfDaemonStatus retval;

	filename = tf_daemon_get_filename ();
	
	if (access (filename, F_OK) != -1) {
		retval = TF_DAEMON_RUNNING;
	} else {
		retval = TF_DAEMON_STOPPED;
	}

	return retval;
}

void
tf_daemon_start (void)
{
	pid_t pid;

	pid = fork ();
	switch (pid) {
	case -1:
		/* Error */
		fprintf (stderr, "Error creating new process\n");
		break;
	case 0:
		execl (TF_LAUNCHER, TF_LAUNCHER, NULL);
		exit (1);
	default: {
		int ret;
		
		do {
			ret = waitpid (pid, NULL, 0);
		} while (ret < 0 && errno == EINTR);
	}
		break;
	}
}

void
tf_daemon_stop (void)
{
	FILE       *fd = NULL;
	const char *filename;
	char        buffer[10];
	char       *p = NULL;
	pid_t       pid;

	filename = tf_daemon_get_filename ();

	fd = fopen (filename, "r");
	if (!fd)
		return;

	p = fgets (buffer, sizeof (buffer), fd);
	if (!p) {
		fclose (fd);
		return;
	}

	fclose (fd);

	pid = atoi (buffer);

	kill (pid, SIGTERM);
}
