/*
 *  Copyright (C) 2006 Carlos Garcia Campos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Authors : Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
 */

#include <glib.h>
#include "tf-event-it.h"

static TfEventClass *parent_class = NULL;

static void tf_event_it_class_init (TfEventItClass *klass);

static gboolean tf_event_it_compare (TfEvent       *ev1,
				     TfEvent       *ev2);

GType
tf_event_it_get_type (void)
{
	static GType type = 0;
	
	if (!type) {
		static const GTypeInfo info = {
			sizeof (TfEventItClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) tf_event_it_class_init,
			NULL,
			NULL,
			sizeof (TfEventIt),
			0,
			(GInstanceInitFunc) NULL
		};
		
		type = g_type_register_static (TF_TYPE_EVENT, "TfEventIt",
					       &info, 0);
	}
	
	return type;
}

static void
tf_event_it_class_init (TfEventItClass *klass)
{
	TfEventClass *event_class = TF_EVENT_CLASS (klass);
	
	parent_class = g_type_class_peek_parent (klass);

	event_class->compare = tf_event_it_compare;
}

TfEvent *
tf_event_it_new (void)
{
	return TF_EVENT (g_object_new (TF_TYPE_EVENT_IT, NULL));
}

static gboolean
tf_event_it_compare (TfEvent *ev1, TfEvent *ev2)
{
	if (!TF_IS_EVENT_IT (ev1))
		return FALSE;

	if (!TF_IS_EVENT_IT (ev2))
		return FALSE;

	/* Two Inactive Time events are always
	 * equal since we don't want to register
	 * two followed IT events
	 */

	return TRUE;
}
