/*
 *  Copyright (C) 2006 Carlos Garcia Campos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Authors : Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
 */

#include <glib.h>
#include "tf-event-xml.h"
#include "tf-event-session-xml.h"

static TfEventSessionClass *parent_class = NULL;

static void   tf_event_session_xml_class_init     (TfEventSessionXmlClass *klass);
static void   tf_event_session_xml_event_xml_init (TfEventXmlIface        *iface);

static gchar *tf_event_session_xml_get_xml        (TfEventXml             *event);


GType
tf_event_session_xml_get_type (void)
{
	static GType type = 0;
	
	if (!type) {
		static const GTypeInfo info = {
			sizeof (TfEventSessionXmlClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) tf_event_session_xml_class_init,
			NULL,
			NULL,
			sizeof (TfEventSessionXml),
			0,
			(GInstanceInitFunc) NULL
		};

		static const GInterfaceInfo register_info = {
			(GInterfaceInitFunc) tf_event_session_xml_event_xml_init,
			NULL,
			NULL
		};
		
		type = g_type_register_static (TF_TYPE_EVENT_SESSION, "TfEventSessionXml",
					       &info, 0);
		
		g_type_add_interface_static (type, TF_TYPE_EVENT_XML,
					     &register_info);
	}
	
	return type;
}

static void
tf_event_session_xml_class_init (TfEventSessionXmlClass *klass)
{
	parent_class = g_type_class_peek_parent (klass);
}

static void
tf_event_session_xml_event_xml_init (TfEventXmlIface *iface)
{
	iface->get_xml = tf_event_session_xml_get_xml;
}

TfEventSessionXml *
tf_event_session_xml_new (TfEventSessionType type)
{
	return TF_EVENT_SESSION_XML (g_object_new (TF_TYPE_EVENT_SESSION_XML,
						   "type", type,
						   NULL));
}

static gchar *
tf_event_session_xml_get_xml (TfEventXml *event)
{
	TfEventSessionType type;
	gulong             time;
	gchar             *xml;

	g_object_get (G_OBJECT (event),
		      "timestamp", &time,
		      "type", &type,
		      NULL);
	
	xml = g_markup_printf_escaped ("\t<event time=\"%ld\" "
				       "type=\"Session\" event=\"%s\" />\n",
				       time,
				       type == TF_EVENT_SESSION_INIT ? "Init" : "End");
	return xml;
}

