/*
 *  Copyright (C) 2006 Carlos Garcia Campos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Authors : Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
 */

#include <glib.h>
#include "tf-event-session.h"

enum {
	PROP_0,
	PROP_TYPE,
};

struct _TfEventSessionPrivate {
	TfEventSessionType type;
};

#define TF_EVENT_SESSION_GET_PRIVATE(object) \
        (G_TYPE_INSTANCE_GET_PRIVATE ((object), TF_TYPE_EVENT_SESSION, TfEventSessionPrivate))

static TfEventClass *parent_class = NULL;

static void tf_event_session_init         (TfEventSession      *event);
static void tf_event_session_class_init   (TfEventSessionClass *klass);

static void tf_event_session_get_property (GObject        *object,
					   guint           prop_id,
					   GValue         *value,
					   GParamSpec     *pspec);
static void tf_event_session_set_property (GObject        *object,
					   guint           prop_id,
					   const GValue   *value,
					   GParamSpec     *pspec);

static gboolean tf_event_session_compare  (TfEvent        *ev1,
					   TfEvent        *ev2);

GType
tf_event_session_get_type (void)
{
	static GType type = 0;
	
	if (!type) {
		static const GTypeInfo info = {
			sizeof (TfEventSessionClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) tf_event_session_class_init,
			NULL,
			NULL,
			sizeof (TfEventSession),
			0,
			(GInstanceInitFunc) tf_event_session_init
		};
		
		type = g_type_register_static (TF_TYPE_EVENT, "TfEventSession",
					       &info, 0);
	}
	
	return type;
}

GType
tf_event_session_type_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GEnumValue values[] = {
			{ TF_EVENT_SESSION_INIT, "TF_EVENT_SESSION_INIT", "init" },
			{ TF_EVENT_SESSION_END,  "TF_EVENT_SESSION_END",  "end" },
			{ 0, NULL, NULL }
		};

		type = g_enum_register_static ("TfEventSessionType", values);
	}

	return type;
}

static void
tf_event_session_init (TfEventSession *event)
{
	event->priv = TF_EVENT_SESSION_GET_PRIVATE (event);

	event->priv->type = TF_EVENT_SESSION_INIT;
}

static void
tf_event_session_class_init (TfEventSessionClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	TfEventClass *event_class = TF_EVENT_CLASS (klass);

	object_class->get_property = tf_event_session_get_property;
	object_class->set_property = tf_event_session_set_property;

     	event_class->compare = tf_event_session_compare;

	parent_class = g_type_class_peek_parent (klass);

	g_type_class_add_private (klass, sizeof (TfEventSessionPrivate));

	g_object_class_install_property (object_class,
					 PROP_TYPE,
					 g_param_spec_enum ("type",
							    "Type",
							    "Event Session Type",
							    TF_TYPE_EVENT_SESSION_TYPE,
							    TF_EVENT_SESSION_INIT,
							    G_PARAM_READWRITE |
							    G_PARAM_CONSTRUCT_ONLY));
}

static void
tf_event_session_get_property (GObject    *object,
			       guint       prop_id,
			       GValue     *value,
			       GParamSpec *pspec)
{
	TfEventSession *event = TF_EVENT_SESSION (object);

	switch (prop_id) {
	case PROP_TYPE:
		g_value_set_enum (value, event->priv->type);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}
}

static void
tf_event_session_set_property (GObject       *object,
			       guint          prop_id,
			       const GValue  *value,
			       GParamSpec    *pspec)
{
	TfEventSession *event = TF_EVENT_SESSION (object);

	switch (prop_id) {
	case PROP_TYPE:
		event->priv->type = g_value_get_enum (value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}
}

TfEvent *
tf_event_session_new (TfEventSessionType type)
{
	return TF_EVENT (g_object_new (TF_TYPE_EVENT_SESSION,
				       "type", type,
				       NULL));
}

static gboolean
tf_event_session_compare (TfEvent *ev1, TfEvent *ev2)
{
	if (!TF_IS_EVENT_SESSION (ev1))
		return FALSE;

	if (!TF_IS_EVENT_SESSION (ev2))
		return FALSE;

	return TF_EVENT_SESSION (ev1)->priv->type == TF_EVENT_SESSION (ev2)->priv->type;
}
