/*
 *  Copyright (C) 2006 Carlos Garcia Campos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Authors : Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
 */

#include <glib.h>
#include <time.h>
#include "tf-event.h"

enum {
	PROP_0,
	PROP_TIMESTAMP
};

struct _TfEventPrivate {
	time_t timestamp;
};

#define TF_EVENT_GET_PRIVATE(object) \
        (G_TYPE_INSTANCE_GET_PRIVATE ((object), TF_TYPE_EVENT, TfEventPrivate))

static GObjectClass *parent_class = NULL;

static void tf_event_init         (TfEvent      *event);
static void tf_event_class_init   (TfEventClass *klass);

static void tf_event_get_property (GObject      *object,
				   guint         prop_id,
				   GValue       *value,
				   GParamSpec   *pspec);

GType
tf_event_get_type (void)
{
	static GType type = 0;
	
	if (!type) {
		static const GTypeInfo info = {
			sizeof (TfEventClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) tf_event_class_init,
			NULL,
			NULL,
			sizeof (TfEvent),
			0,
			(GInstanceInitFunc) tf_event_init
		};
		
		type = g_type_register_static (G_TYPE_OBJECT, "TfEvent",
					       &info, G_TYPE_FLAG_ABSTRACT);
	}
	
	return type;
}

static void
tf_event_init (TfEvent *event)
{
	event->priv = TF_EVENT_GET_PRIVATE (event);

	event->priv->timestamp = time (NULL);
}

static void
tf_event_class_init (TfEventClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	klass->compare = NULL;
	
	object_class->get_property = tf_event_get_property;

	parent_class = g_type_class_peek_parent (klass);

	g_type_class_add_private (klass, sizeof (TfEventPrivate));

	g_object_class_install_property (object_class,
					 PROP_TIMESTAMP,
					 g_param_spec_ulong ("timestamp",
							     "TimeStamp",
							     "TimeStamp of the event",
							     0, G_MAXULONG, 0,
							     G_PARAM_READABLE));
}

static void
tf_event_get_property (GObject    *object,
		       guint       prop_id,
		       GValue     *value,
		       GParamSpec *pspec)
{
	TfEvent *event = TF_EVENT (object);

	switch (prop_id) {
	case PROP_TIMESTAMP:
		g_value_set_ulong (value, event->priv->timestamp);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}
}

gboolean
tf_event_compare (TfEvent *ev1, TfEvent *ev2)
{
	TfEventClass *klass;
	
	g_return_val_if_fail (TF_IS_EVENT (ev1), FALSE);
	g_return_val_if_fail (TF_IS_EVENT (ev2), FALSE);

	klass = TF_EVENT_GET_CLASS (ev1);
	
	if (klass->compare)
		return (* klass->compare) (ev1, ev2);

	return FALSE;
}
