/*
 *  Copyright (C) 2006 Carlos Garcia Campos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Authors : Carlos Garcia Campos <carlosgc@gsyc.escet.urjc.es>
 */

#include "tf-register.h"

static void tf_register_base_init (gpointer g_iface);

GType
tf_register_get_type (void)
{
	static GType type = 0;
	
	if (!type) {
		static const GTypeInfo info = {
			sizeof (TfRegisterIface),
			tf_register_base_init,
			NULL,
			NULL,
			NULL,
			NULL,
			0,
			0,
			NULL
		};
		
		type = g_type_register_static (G_TYPE_INTERFACE, "TfRegister",
					       &info, 0);
		
		g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
	}
	
	return type;
}

static void
tf_register_base_init (gpointer g_iface)
{
	static gboolean initialized = FALSE;
	
	if (!initialized) {
		initialized = TRUE;
	}
}

void
tf_register_save_event (TfRegister *reg, const TfEvent *event)
{
	TfRegisterIface *iface;
	
	g_return_if_fail (TF_IS_REGISTER (reg));
	g_return_if_fail (TF_IS_EVENT (event));

	iface = TF_REGISTER_GET_IFACE (reg);
	
	if (iface->save_event)
		(* iface->save_event) (reg, event);
}

gboolean
tf_register_lock (TfRegister *reg)
{
	TfRegisterIface *iface;
	
	g_return_val_if_fail (TF_IS_REGISTER (reg), TRUE);

	iface = TF_REGISTER_GET_IFACE (reg);

	if (iface->lock)
		return (* iface->lock) (reg);

	return TRUE;
}

void
tf_register_unlock (TfRegister *reg)
{
	TfRegisterIface *iface;

	g_return_if_fail (TF_IS_REGISTER (reg));

	iface = TF_REGISTER_GET_IFACE (reg);

	if (iface->unlock)
		(* iface->unlock) (reg);
}
