#!/usr/bin/perl
#
#
# Tempus Fugit Data Analyzer
#
# Autor: Alvaro del Castillo <acs@gsyc.es>
#
# GPL v2 or newer
#
# <events>
#   <event time="1152028941" type="WindowChange">
#		<app>Gnome-terminal</app>
#		<title>root@delito: /home/acs/devel/libresoft/detras</title>
#   </event>
# </events>

use XML::LibXML;
use Data::Dumper;

use warnings;
use strict;


my $filename = $ENV{'HOME'}."/.detras/tempusfugit.xml";
my $parser = XML::LibXML->new();
my $tree = $parser->parse_file($filename);
my $root = $tree->getDocumentElement;

my $start = 0;
my $stamp = 0;
my $delta = $stamp-$start;

my $applications = {};
my $app_title;
my $app_name;

my @events = $root->getElementsByTagName('event');
print "Total events: " . @events ."\n";
foreach my $event (@events) {    
    my @atributos = $event->attributes();
    if ($app_name) {
        $delta = $atributos[0]->value-$stamp;
        $applications->{$app_name} += $delta;
        $app_name = "Unknown";
    }

    $stamp = $atributos[0]->value;
    $start = $stamp if $start == 0;

    my @apps = $event->getElementsByTagName('app');
    my @titles = $event->getElementsByTagName('title');
    
    if ($apps[0]) {  
        $app_name = $apps[0]->textContent;
        $app_title = $titles[0];
    }

    if (!$app_name) {
        $app_name = "Unknown";
    }
}

# Final report
my $total_time = int(($stamp - $start)/60);
print "Total time recorded: $total_time minutes\n";
print "Total number of events: ". @events ."\n";
if ($total_time>0) {
    print "Total number of events per minute: " . eval(@events/$total_time)."\n\n";
}

# Applications sorted by time
foreach my $app (sort {$applications->{$b} <=> $applications->{$a}} 
                 keys %$applications) {
    print "- $app ". $applications->{$app} . " secs\n";
}
