#!/usr/bin/perl
#
#
# Tempus Fugit Data Analyzer with Gtk+ interface
#
# Autor: Alvaro del Castillo <acs@gsyc.es>
#
# GPL v2 or newer
#
# <events>
#   <event time="1152028941" type="WindowChange">
#		<app>Gnome-terminal</app>
#		<title>root@delito: /home/acs/devel/libresoft/detras</title>
#   </event>
# </events>

use warnings;
use strict;

use XML::LibXML;

use Glib qw(TRUE FALSE);
use Gtk2 '-init'; # auto-initializes Gtk2
use Gtk2::GladeXML;

use Data::Dumper;

# Columns Application and Time
use constant COLUMN_APP  => 0;
use constant COLUMN_TIME => 1;


sub gtk_main_quit {
    Gtk2->main_quit;
}

sub create_model_apps {
    my ($applications) = @_;

    my $store = Gtk2::ListStore->new ('Glib::String',
                                      'Glib::String');

    foreach my $app (keys %$applications) {
        my $iter = $store->append;
        $store->set ($iter,
                     COLUMN_APP, $app,
                     COLUMN_TIME, $applications->{$app},
                     );
    }
    return $store;
}

sub column_compare_number  {
    my ($model, $iter1, $iter2, $data) = @_;

    my $number1 = $model->get ($iter1, COLUMN_TIME);
    my $number2 = $model->get ($iter2, COLUMN_TIME);

    my $res = 0;

    if ($number1 < $number2) {
        $res = 1;
    } elsif ($number1 > $number2) {
        $res = -1;
    }

    return $res;    
}

sub add_columns_apps {
    my ($treeview) = @_;

    my $model = $treeview->get_model;

    my $renderer = Gtk2::CellRendererText->new;
    my $column = Gtk2::TreeViewColumn->new_with_attributes ("Application",
                                                            $renderer,
                                                            text   => COLUMN_APP);
    
    $column->set_sort_column_id (COLUMN_APP);
    $treeview->append_column ($column);
    
    $renderer = Gtk2::CellRendererText->new;
    $column = Gtk2::TreeViewColumn->new_with_attributes ("Time (secs)",
                                                         $renderer,
                                                         text   => COLUMN_TIME);
    $column->set_sort_column_id (COLUMN_TIME);
    $model->set_sort_func (COLUMN_TIME, \&column_compare_number, undef);
    $treeview->append_column ($column);
}

# GUI building

my $gui;
$gui->{glade} = Gtk2::GladeXML->new("GAnalTempusFugit.glade");
$gui->{glade}->signal_autoconnect_from_package('main');

$gui->{window} = $gui->{glade}->get_widget('window1');
$gui->{treeview} = $gui->{glade}->get_widget('treeview1');

my $filename = $ENV{'HOME'}."/.detras/tempusfugit.xml";
my $parser = XML::LibXML->new();
my $tree = $parser->parse_file($filename);
my $root = $tree->getDocumentElement;

my $number_ev = 0;
my $start = 0;
my $stamp = 0;
my $delta = $stamp-$start;

my $applications = {};

my @events = $root->getElementsByTagName('event');
my ($app_name, $app_title); 
foreach my $event (@events) {    
    my @atributos = $event->attributes();
    if ($app_name) {
        $delta = $atributos[0]->value-$stamp;
        $applications->{$app_name} += $delta;
        $app_name = "Unknown";
    }

    $stamp = $atributos[0]->value;
    $start = $stamp if $start == 0;

    my @apps = $event->getElementsByTagName('app');
    my @titles = $event->getElementsByTagName('title');
    
    if ($apps[0]) {  
        $app_name = $apps[0]->textContent;
        $app_title = $titles[0];
    }

    if (!$app_name) {
        $app_name = "Unknown";
    }
}


my $model_apps = create_model_apps ($applications);
$gui->{treeview}->set_model ($model_apps);
add_columns_apps ($gui->{treeview});

Gtk2->main;

exit 0;
