#!/usr/bin/python2.4
#
#
# Tempus Fugit Data Analyzer with Gtk+ interface
#
# Autor: Alvaro del Castillo <acs@gsyc.es>
#
# GPL v2 or newer
#
# <events>
#   <event time="1152028941" type="WindowChange">
#		<app>Gnome-terminal</app>
#		<title>root@delito: /home/acs/devel/libresoft/detras</title>
#   </event>
# </events>

import sys
import os

import libxml2
import pprint

import gobject
import gtk
import gtk.glade

(COLUMN_APP, COLUMN_TIME) = range (2)

def check_time (apps_time, total_time):
    total_time_apps = 0;
    for app in apps_time:
        total_time_apps += apps_time[app]
    print "Total time: " + str(total_time_apps) + "/" + str(total_time)
    if (total_time_apps == total_time):
        return True
    else:
        return False

def gtk_main_quit(*args):
    gtk.main_quit()

def create_model_apps(apps_time):
    store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_UINT)
    for app in apps_time:
        iter = store.append();
        store.set(iter,
                  COLUMN_APP, app,
                  COLUMN_TIME, apps_time[app])
    return store;
    

def add_columns_apps(treeview):
    cell = gtk.CellRendererText()
    column = gtk.TreeViewColumn('Application', cell, text=COLUMN_APP)
    column.set_sort_column_id(COLUMN_APP)
    column.set_expand(True)
    treeview.append_column(column)

    cell = gtk.CellRendererText()
    column = gtk.TreeViewColumn('Time (secs)', cell, text=COLUMN_TIME)
    column.set_sort_column_id(COLUMN_TIME)
    column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
    column.set_fixed_width(100)
    treeview.append_column(column)    

def parse_tempusfile ():
    tempusfile  = os.environ["HOME"]+"/.detras/tempusfugit.xml";

    doc = libxml2.parseFile(tempusfile)
    if doc.name != tempusfile:
        print tempusfile +" failed"
        sys.exit(1)

    root = doc.children
    if root.name != "events":
        print "root.name failed. XML file malformed" + tempusfile
        sys.exit(1)

    number_ev  = 0
    start      = 0
    stamp      = 0
    eventStamp = 0
    delta      = stamp-start
    apps_time  = {}
    app_name   = ""
    app_title  = ""

    # reach all subnodes also
    for child in root:
        if child.type == "element":
            if (child.name == "app"):
                app_name = child.content
            elif (child.name == "title"):
                app_title = child.content        
            elif (child.name == "event"):
                number_ev += 1
                prop = child.hasProp ("time")
                eventStamp = int(prop.content)
                if (start == 0):
                    start = eventStamp
                elif (eventStamp > stamp):
                    delta = eventStamp - stamp
                    # print "Delta " + app_name + ":" + str(delta)
                    if  (app_name != ""):
                        try:
                            apps_time[app_name] += delta
                        except KeyError:
                            apps_time[app_name] = delta
                    else:
                       try:
                           apps_time['unknown'] += delta
                       except KeyError:
                           apps_time['unknown'] = delta 
                stamp = eventStamp
                app_name = app_title = ""
                # print "Element " + child.name + " " + \
                #      app_name + " " + app_title+" " +str(delta)

    # pp.pprint (apps_time)
    total_time = stamp - start
    print "Total time: " + str(total_time/60)
    print "Number events: " + str (number_ev)
    if (check_time (apps_time, total_time)):
        print "Time computing OK"
    doc.freeDoc()
    return apps_time


def main():
    # pp = pprint.PrettyPrinter(indent=4)

    if len(sys.argv) > 1:
        fname = sys.argv[1]
    else:
        fname = 'GAnalTempusFugit.glade'

    xml = gtk.glade.XML(fname)
    xml.signal_autoconnect(locals())
    treeview = xml.get_widget('treeview1')
        
    # Data model
    apps_time = parse_tempusfile ()

    model_apps = create_model_apps (apps_time);
    treeview.set_model (model_apps);
    add_columns_apps (treeview);

    gtk.main()


if __name__ == '__main__':
    main()
